@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_pixels.c,v 1.9 2003/08/27 04:53:40 slouken Exp $";
+ "@(#) $Id: SDL_pixels.c,v 1.12 2004/11/30 14:28:20 slouken Exp $";
 #endif
 
 /* General (mostly internal) pixel/color manipulation routines for SDL */
@@ -246,7 +246,12 @@ SDL_PixelFormat *SDL_ReallocFormat(SDL_Surface *surface, int bpp,
  */
 void SDL_FormatChanged(SDL_Surface *surface)
 {
-	surface->format_version++;
+	static int format_version = 0;
+	++format_version;
+	if ( format_version < 0 ) { /* It wrapped... */
+		format_version = 1;
+	}
+	surface->format_version = format_version;
 	SDL_InvalidateMap(surface->map);
 }
 /*
@@ -489,6 +494,10 @@ static Uint8 *MapNto1(SDL_PixelFormat *src, SDL_Palette *dst, int *identical)
 	/* Generate a 256 color dither palette */
 	SDL_Palette dithered;
 	SDL_Color colors[256];
+	
+	/* SDL_DitherColors does not initialize the 'unused' component of colors,
+	   but Map1to1 compares it against dst, so we should initialize it. */  
+	memset(colors, 0, sizeof(colors));
 
 	dithered.ncolors = 256;
 	SDL_DitherColors(colors, 8);
