@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_syswm.cc,v 1.5 2004/01/04 16:49:24 slouken Exp $";
+ "@(#) $Id: SDL_syswm.cc,v 1.6 2004/07/18 19:14:33 slouken Exp $";
 #endif
 
 #include "SDL_BWin.h"
@@ -30,10 +30,30 @@ static char rcsid =
 extern "C" {
 
 #include "SDL_syswm_c.h"
+#include "SDL_error.h"
 
 void BE_SetWMCaption(_THIS, const char *title, const char *icon)
 {
 	SDL_Win->SetTitle(title);
 }
 
+int BE_IconifyWindow(_THIS)
+{
+	SDL_Win->Minimize(true);
+}
+
+int BE_GetWMInfo(_THIS, SDL_SysWMinfo *info)
+{
+    if (info->version.major <= SDL_MAJOR_VERSION)
+    {
+        return 1;
+    }
+    else
+    {
+        SDL_SetError("Application not compiled with SDL %d.%d\n",
+                      SDL_MAJOR_VERSION, SDL_MINOR_VERSION);
+        return -1;
+    }
+}
+
 }; /* Extern C */
