@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_gemvideo.c,v 1.12 2004/02/06 22:53:32 pmandin Exp $";
+ "@(#) $Id: SDL_gemvideo.c,v 1.30 2004/11/27 21:28:49 pmandin Exp $";
 #endif
 
 /*
@@ -47,6 +47,7 @@ static char rcsid =
 #include "SDL_error.h"
 #include "SDL_video.h"
 #include "SDL_mouse.h"
+#include "SDL_endian.h"
 #include "SDL_sysvideo.h"
 #include "SDL_pixels_c.h"
 #include "SDL_events_c.h"
@@ -55,6 +56,8 @@ static char rcsid =
 #include "SDL_ataric2p_s.h"
 #include "SDL_atarieddi_s.h"
 #include "SDL_atarimxalloc_c.h"
+#include "SDL_atarigl_c.h"
+
 #include "SDL_gemvideo.h"
 #include "SDL_gemevents_c.h"
 #include "SDL_gemmouse_c.h"
@@ -63,7 +66,7 @@ static char rcsid =
 
 /* Defines */
 
-/* #define DEBUG_VIDEO_GEM	1 */
+/*#define DEBUG_VIDEO_GEM	1*/
 
 #define GEM_VID_DRIVER_NAME "gem"
 
@@ -107,6 +110,11 @@ static void GEM_LockScreen(_THIS);
 static void GEM_UnlockScreen(_THIS);
 static void refresh_window(_THIS, int winhandle, short *rect);
 
+#ifdef HAVE_OPENGL
+/* OpenGL functions */
+static void GEM_GL_SwapBuffers(_THIS);
+#endif
+
 /* GEM driver bootstrap functions */
 
 static int GEM_Available(void)
@@ -135,6 +143,8 @@ static SDL_VideoDevice *GEM_CreateDevice(int devindex)
 		memset(device, 0, (sizeof *device));
 		device->hidden = (struct SDL_PrivateVideoData *)
 				malloc((sizeof *device->hidden));
+		device->gl_data = (struct SDL_PrivateGLData *)
+				malloc((sizeof *device->gl_data));
 	}
 	if ( (device == NULL) || (device->hidden == NULL) ) {
 		SDL_OutOfMemory();
@@ -144,6 +154,7 @@ static SDL_VideoDevice *GEM_CreateDevice(int devindex)
 		return(0);
 	}
 	memset(device->hidden, 0, (sizeof *device->hidden));
+	memset(device->gl_data, 0, sizeof(*device->gl_data));
 
 	/* Set the function pointers */
 	device->VideoInit = GEM_VideoInit;
@@ -176,6 +187,15 @@ static SDL_VideoDevice *GEM_CreateDevice(int devindex)
 	device->WarpWMCursor = NULL /*GEM_WarpWMCursor*/;
 	device->CheckMouseMode = GEM_CheckMouseMode;
 
+#ifdef HAVE_OPENGL
+	/* OpenGL functions */
+	device->GL_LoadLibrary = SDL_AtariGL_LoadLibrary;
+	device->GL_GetProcAddress = SDL_AtariGL_GetProcAddress;
+	device->GL_GetAttribute = SDL_AtariGL_GetAttribute;
+	device->GL_MakeCurrent = SDL_AtariGL_MakeCurrent;
+	device->GL_SwapBuffers = GEM_GL_SwapBuffers;
+#endif
+
 	/* Joystick + Mouse relative motion */
 	SDL_AtariXbios_InstallVectors(ATARI_XBIOS_MOUSEEVENTS|ATARI_XBIOS_JOYSTICKEVENTS);
 
@@ -293,27 +313,11 @@ int GEM_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	if (GEM_version >= 0x0410) {
 		short ap_gout[4], errorcode;
 		
-#ifdef DEBUG_VIDEO_GEM
-		printf("sdl:video:gem: AES %02x.%02x\n", (GEM_version>>8) & 0xff, GEM_version & 0xff);
-#endif
-
 		GEM_wfeatures=0;
 		errorcode=appl_getinfo(AES_WINDOW, &ap_gout[0], &ap_gout[1], &ap_gout[2], &ap_gout[3]);
-#ifdef DEBUG_VIDEO_GEM
-		printf("sdl:video:gem: appl_getinfo() returned 0x%04x\n", errorcode);
-#endif
 
 		if (errorcode==0) {
 			GEM_wfeatures=ap_gout[0];			
-
-#ifdef DEBUG_VIDEO_GEM
-			printf("sdl:video:gem: AES wind_*() modes: 0x%04x\n", GEM_wfeatures);
-			printf("sdl:video:gem: AES window behaviours: 0x%04x\n", ap_gout[3]);
-		} else {
-			printf("sdl:video:gem: apgout[]={0x%04x,0x%04x,0x%04x,0x%04x}\n",
-				ap_gout[0], ap_gout[1], ap_gout[1], ap_gout[3]
-			);
-#endif
 		}
 	}	
 
@@ -400,6 +404,8 @@ int GEM_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	GEM_handle = -1;
 	GEM_locked = SDL_FALSE;
 	GEM_win_fulled = SDL_FALSE;
+	GEM_fullscreen = SDL_FALSE;
+	GEM_lock_redraw = SDL_TRUE;	/* Prevent redraw till buffers are setup */
 
 	VDI_screen = NULL;
 	VDI_pitch = VDI_w * VDI_pixelsize;
@@ -437,8 +443,17 @@ int GEM_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	vsf_interior(VDI_handle,1);
 	vsf_perimeter(VDI_handle,0);
 
-#ifdef DEBUG_VIDEO_GEM
-	printf("sdl:video:gem: VideoInit(): done\n");
+	/* Fill video modes list */
+	SDL_modelist[0] = malloc(sizeof(SDL_Rect));
+	SDL_modelist[0]->x = 0;
+	SDL_modelist[0]->y = 0;
+	SDL_modelist[0]->w = VDI_w;
+	SDL_modelist[0]->h = VDI_h;
+
+	SDL_modelist[1] = NULL;
+
+#ifdef HAVE_OPENGL
+	SDL_AtariGL_InitPointers(this);
 #endif
 
 	/* We're done! */
@@ -447,11 +462,15 @@ int GEM_VideoInit(_THIS, SDL_PixelFormat *vformat)
 
 SDL_Rect **GEM_ListModes(_THIS, SDL_PixelFormat *format, Uint32 flags)
 {
-	if (format->BitsPerPixel == VDI_bpp) {
-		return((SDL_Rect **)-1);
-	} else {
+	if (format->BitsPerPixel != VDI_bpp) {
 		return ((SDL_Rect **)NULL);
 	}
+
+	if (flags & SDL_FULLSCREEN) {
+		return (SDL_modelist);
+	}
+
+	return((SDL_Rect **)-1);
 }
 
 static void GEM_FreeBuffers(_THIS)
@@ -466,13 +485,6 @@ static void GEM_FreeBuffers(_THIS)
 		free( GEM_buffer1 );
 		GEM_buffer1=NULL;
 	}
-
-	/* Destroy window */
-	if (GEM_handle>=0) {
-		wind_close(GEM_handle);
-		wind_delete(GEM_handle);
-		GEM_handle=-1;
-	}
 }
 
 static void GEM_ClearRect(_THIS, short *rect)
@@ -507,11 +519,11 @@ static void GEM_ClearScreen(_THIS)
 static void GEM_LockScreen(_THIS)
 {
 	if (!GEM_locked) {
-		/* Reserve memory space, used to be sure of compatibility */
-		form_dial( FMD_START, 0,0,0,0, 0,0,VDI_w,VDI_h);
 		/* Lock AES */
 		wind_update(BEG_UPDATE);
 		wind_update(BEG_MCTRL);
+		/* Reserve memory space, used to be sure of compatibility */
+		form_dial( FMD_START, 0,0,0,0, 0,0,VDI_w,VDI_h);
 
 		GEM_locked=SDL_TRUE;
 	}
@@ -537,8 +549,6 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 	Uint32 modeflags, screensize;
 	SDL_bool use_shadow1, use_shadow2;
 
-	GEM_FreeBuffers(this);
-
 	/*--- Verify if asked mode can be used ---*/
 	if (flags & SDL_FULLSCREEN) {
 		maxwidth=VDI_w;
@@ -550,7 +560,7 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 	}
 
 	/* width must be multiple of 16, for vro_cpyfm() and c2p_convert() */
-	if ((width & -16) != 0) {
+	if ((width & 15) != 0) {
 		width = (width | 15) +1;
 	}
 
@@ -572,6 +582,8 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 #endif
 
 	/*--- Allocate shadow buffers if needed, and conversion operations ---*/
+	GEM_FreeBuffers(this);
+
 	GEM_bufops=0;
 	use_shadow1=use_shadow2=SDL_FALSE;
 	if (VDI_screen && (flags & SDL_FULLSCREEN)) {
@@ -592,7 +604,7 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 	if (use_shadow1) {
 		GEM_buffer1 = Atari_SysMalloc(screensize, MX_PREFTTRAM);
 		if (GEM_buffer1==NULL) {
-			fprintf(stderr,"Unable to allocate shadow buffer\n");
+			SDL_SetError("Unable to allocate shadow buffer");
 			return NULL;
 		}
 		memset(GEM_buffer1, 0, screensize);
@@ -604,7 +616,7 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 	if (use_shadow2) {
 		GEM_buffer2 = Atari_SysMalloc(screensize, MX_PREFTTRAM);
 		if (GEM_buffer2==NULL) {
-			fprintf(stderr,"Unable to allocate shadow buffer\n");
+			SDL_SetError("Unable to allocate shadow buffer");
 			return NULL;
 		}
 		memset(GEM_buffer2, 0, screensize);
@@ -630,21 +642,16 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 		} else {
 			modeflags |= SDL_SWSURFACE;
 		}
+
+		GEM_fullscreen = SDL_TRUE;
 	} else {
-		int posx,posy;
+		int old_win_type;
 		short x2,y2,w2,h2;
 
 		GEM_UnlockScreen(this);
 
-		/* Center our window */
-		posx = GEM_desk_x;
-		posy = GEM_desk_y;
-		if (width<GEM_desk_w)
-			posx += (GEM_desk_w - width) >> 1;
-		if (height<GEM_desk_h)
-			posy += (GEM_desk_h - height) >> 1;
-
-		/* Calculate our window size and position */
+		/* Set window gadgets */
+		old_win_type = GEM_win_type;
 		if (!(flags & SDL_NOFRAME)) {
 			GEM_win_type=NAME|MOVER|CLOSER|SMALLER;
 			if (flags & SDL_RESIZABLE) {
@@ -655,37 +662,51 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 			GEM_win_type=0;
 			modeflags |= SDL_NOFRAME;
 		}
+		modeflags |= SDL_SWSURFACE;
 
-		if (!wind_calc(0, GEM_win_type, posx, posy, width, height, &x2, &y2, &w2, &h2)) {
-			GEM_FreeBuffers(this);
-			fprintf(stderr,"Can not calculate window attributes\n");
-			return NULL;
-		}
+		/* Recreate window ? only for different widget or non-created window */
+		if ((old_win_type != GEM_win_type) || (GEM_handle < 0)) {
+			/* Calculate window size */
+			if (!wind_calc(WC_BORDER, GEM_win_type, 0,0,width,height, &x2,&y2,&w2,&h2)) {
+				GEM_FreeBuffers(this);
+				SDL_SetError("Can not calculate window attributes");
+				return NULL;
+			}
 
-		/* Create window */
-		GEM_handle=wind_create(GEM_win_type, x2, y2, w2, h2);
-		if (GEM_handle<0) {
-			GEM_FreeBuffers(this);
-			fprintf(stderr,"Can not create window\n");
-			return NULL;
-		}
+			/* Center window */
+			x2 = GEM_desk_x+((GEM_desk_w-w2)>>1);
+			y2 = GEM_desk_y+((GEM_desk_h-h2)>>1);
+
+			/* Destroy existing window */
+			if (GEM_handle >= 0) {
+				wind_close(GEM_handle);
+				wind_delete(GEM_handle);
+			}
+
+			/* Create window */
+			GEM_handle=wind_create(GEM_win_type, x2,y2,w2,h2);
+			if (GEM_handle<0) {
+				GEM_FreeBuffers(this);
+				SDL_SetError("Can not create window");
+				return NULL;
+			}
 
 #ifdef DEBUG_VIDEO_GEM
-		printf("sdl:video:gem: handle=%d\n", GEM_handle);
+			printf("sdl:video:gem: handle=%d\n", GEM_handle);
 #endif
 
-		/* Setup window name */
-		wind_set(GEM_handle,WF_NAME,(short)(((unsigned long)GEM_title_name)>>16),(short)(((unsigned long)GEM_title_name) & 0xffff),0,0);
-		GEM_refresh_name = SDL_FALSE;
-	
-		/* Open the window */
-		wind_open(GEM_handle,x2,y2,w2,h2);
+			/* Setup window name */
+			wind_set(GEM_handle,WF_NAME,(short)(((unsigned long)GEM_title_name)>>16),(short)(((unsigned long)GEM_title_name) & 0xffff),0,0);
+			GEM_refresh_name = SDL_FALSE;
 
-		modeflags |= SDL_SWSURFACE;
+			/* Open the window */
+			wind_open(GEM_handle,x2,y2,w2,h2);
+		}
+
+		GEM_fullscreen = SDL_FALSE;
 	}
 
 	/* Set up the new mode framebuffer */
-	current->flags = modeflags;
 	current->w = width;
 	current->h = height;
 	if (use_shadow1) {
@@ -698,11 +719,26 @@ SDL_Surface *GEM_SetVideoMode(_THIS, SDL_Surface *current,
 		current->pitch = VDI_pitch;
 	}
 
+#ifdef HAVE_OPENGL
+	if (flags & SDL_OPENGL) {
+		if (!SDL_AtariGL_Init(this, current)) {
+			GEM_FreeBuffers(this);
+			SDL_SetError("Can not create OpenGL context");
+			return NULL;
+		}
+
+		modeflags |= SDL_OPENGL;
+	}
+#endif
+
+	current->flags = modeflags;
+
 #ifdef DEBUG_VIDEO_GEM
 	printf("sdl:video:gem: surface: %dx%d\n", current->w, current->h);
 #endif
 
 	this->UpdateRects = GEM_UpdateRects;
+	GEM_lock_redraw = SDL_FALSE;	/* Enable redraw */
 
 	/* We're done */
 	return(current);
@@ -736,7 +772,7 @@ static void GEM_UpdateRectsFullscreen(_THIS, int numrects, SDL_Rect *rects)
 	surface = this->screen;
 	/* Need to be a multiple of 16 pixels */
 	surf_width=surface->w;
-	if ((surf_width & -16) != 0) {
+	if ((surf_width & 15) != 0) {
 		surf_width = (surf_width | 15) + 1;
 	}
 
@@ -841,6 +877,10 @@ static void GEM_UpdateRects(_THIS, int numrects, SDL_Rect *rects)
 {
 	SDL_Surface *surface;
 
+	if (GEM_lock_redraw) {
+		return;
+	}
+
 	surface = this->screen;
 
 	if (surface->flags & SDL_FULLSCREEN) {
@@ -856,7 +896,7 @@ static int GEM_FlipHWSurfaceFullscreen(_THIS, SDL_Surface *surface)
 
 	/* Need to be a multiple of 16 pixels */
 	surf_width=surface->w;
-	if ((surf_width & -16) != 0) {
+	if ((surf_width & 15) != 0) {
 		surf_width = (surf_width | 15) + 1;
 	}
 
@@ -934,6 +974,10 @@ static int GEM_FlipHWSurfaceWindowed(_THIS, SDL_Surface *surface)
 
 static int GEM_FlipHWSurface(_THIS, SDL_Surface *surface)
 {
+	if (GEM_lock_redraw) {
+		return(0);
+	}
+
 	if (surface->flags & SDL_FULLSCREEN) {
 		return GEM_FlipHWSurfaceFullscreen(this, surface);
 	} else {
@@ -997,6 +1041,19 @@ void GEM_VideoQuit(_THIS)
 
 	GEM_FreeBuffers(this);
 
+#ifdef HAVE_OPENGL
+	if (gl_active) {
+		SDL_AtariGL_Quit(this, SDL_TRUE);
+	}
+#endif
+
+	/* Destroy window */
+	if (GEM_handle>=0) {
+		wind_close(GEM_handle);
+		wind_delete(GEM_handle);
+		GEM_handle=-1;
+	}
+
 	GEM_UnlockScreen(this);
 
 	appl_exit();
@@ -1187,7 +1244,7 @@ static void refresh_window(_THIS, int winhandle, short *rect)
 
 		/* Need to be a multiple of 16 pixels */
 		width=surface->w;
-		if ((width & -16) != 0) {
+		if ((width & 15) != 0) {
 			width = (width | 15) + 1;
 		}
 		mfdb_src.fd_w=width;
@@ -1214,3 +1271,13 @@ static void refresh_window(_THIS, int winhandle, short *rect)
 
 	vro_cpyfm( VDI_handle, S_ONLY, pxy, &mfdb_src, &VDI_dst_mfdb);
 }
+
+#ifdef HAVE_OPENGL
+
+static void GEM_GL_SwapBuffers(_THIS)
+{
+	SDL_AtariGL_SwapBuffers(this);
+	GEM_FlipHWSurface(this, this->screen);
+}
+
+#endif
